"""
pydiceroller init
"""

from random import random
from pydiceroller.args import compute_args
import os
import random
import subprocess
from shutil import which
import re

DICE_EMOJI = "\U0001f3b2"

def pydiceroller():
    """
    pydiceroller entry point
    """
    #fix potential windows bug
    #colorama.init()
    os.sys.stdin.reconfigure(encoding='utf-8')
    os.sys.stdout.reconfigure(encoding='utf-8')
    if not compute_args().dices:
        print("no dice given")    
        exit(1)

    for dice in compute_args().dices:
        if not re.match("^[1-9][0-9]*d[1-9][0-9]*$",dice.lower() ) and not re.match("^d?[1-9][0-9]*$",dice.lower() ) and not re.match("^s[1-9][0-9]*d[1-9][0-9]*$",dice.lower()) and not re.match("^[1-9][0-9]*s[1-9][0-9]*d[1-9][0-9]*$",dice.lower()):
            print("bad param, use --help to see correct syntax") 
            exit(1)

    while True:
        for dice in compute_args().dices:
            if re.match("^[1-9][0-9]*d[1-9][0-9]*$",dice.lower() ):
                number=dice.lower().split('d')[0]
                diceone=dice.lower().split('d')[1]
                for r in range(int(number)):
                    print(((DICE_EMOJI + diceone + " : ") if not compute_args().silent else "") + str(random.randint(1, int(diceone))))
            elif re.match("^d[1-9][0-9]*$",dice.lower() ):
                number=1
                diceone=dice[1:]
                for r in range(int(number)):
                    print(((DICE_EMOJI + diceone + " : ") if not compute_args().silent else "") + str(random.randint(1, int(diceone))))              
            elif re.match("^[1-9][0-9]*$",dice.lower() ):
                number=1
                diceone=dice
                for r in range(int(number)):
                    print(((DICE_EMOJI + diceone + " : ") if not compute_args().silent else "") + str(random.randint(1, int(diceone))))              

            elif re.match("^s[1-9][0-9]*d[1-9][0-9]*$",dice.lower() ):
                number=dice[1:].lower().split('d')[0]
                diceone=dice[1:].lower().split('d')[1]
                result=0
                for r in range(int(number)):
                    result=result+random.randint(1, int(diceone))
                print(((number + DICE_EMOJI + diceone + " : ") if not compute_args().silent else "")  + str(result)) 
            elif re.match("^[1-9][0-9]*s[1-9][0-9]*d[1-9][0-9]*$",dice.lower() ):
                numberdice=dice.lower().split('s')[0]
                dicee=dice.lower().split('s')[1]
                for r in range(int(numberdice)):
                    number=dicee.lower().split('d')[0]
                    diceone=dicee.lower().split('d')[1]
                    result=0
                    for r in range(int(number)):
                        result=result+random.randint(1, int(diceone))
                    print(((number + DICE_EMOJI + diceone + " : ") if not compute_args().silent else "")  + str(result))                    
        if compute_args().infinite:
            try:
                input("press any key to relaunch dices, ctrl-c to exit")
            except KeyboardInterrupt:
                break    
        else:
            break
   


"""
pygitscrum argparse gestion
"""

import argparse
import sys
from argparse import RawTextHelpFormatter
import importlib.metadata

def get_env_report():
    lines = []

    lines.append("\nInstalled packages:")
    for dist in sorted(importlib.metadata.distributions(), key=lambda d: d.metadata["Name"].lower()):
        name = dist.metadata["Name"]
        version = dist.version
        lines.append(f"  - {name}=={version}")

    return "\n".join(lines)


def compute_args():
    """
    check args and return them
    """
    my_parser = argparse.ArgumentParser(
        description="""
Synopsis : minimal dice roller
        """,
        epilog=f"""
To upgrade, run:
    pipx upgrade pyconwaysgame --include-deps
To install, run:
    pipx install pyconwaysgame
To force reinstall, run:
    pipx install pyconwaysgame --force
To uninstall, run:
    pipx uninstall pyconwaysgame
To force uninstall (if needed), run:
    pipx uninstall pyconwaysgame --force

{get_env_report()}

Full documentation at: <https://github.com/thib1984/pydiceroller>
Report bugs to <https://github.com/thib1984/pydiceroller/issues>
MIT Licence
This is free software: you are free to change and redistribute it
There is NO WARRANTY, to the extent permitted by law
Written by thib1984."""
        , formatter_class=RawTextHelpFormatter
    )  
    my_parser.add_argument(
        "dices",
        metavar="dices",
        type=str,
        nargs="*",
        help="dices, with format example : 6, D6, 2D10, S3D10 (for sum), 2S2D6 -can give one or more parameters-",
    ) 
    my_parser.add_argument(
        "-s",
        "--silent",
        action="store_true",
        help="minimal output, use for pip commands",
    ),
    my_parser.add_argument(
        "-i",
        "--infinite",
        action="store_true",
        help="press entry to relaunch same dices, ctrl-c to exit",
    ),                        

    args = my_parser.parse_args()
    return args



from setuptools import setup


setup(
    name="pydiceroller",
    version="1.1.0",
    description="a minimal dice roller",
    long_description="The complete description/installation/use/FAQ is available at : https://github.com/thib1984/pydiceroller#readme",
    url="https://github.com/thib1984/pydiceroller",
    author="thib1984",
    author_email="thibault.garcon@gmail.com",
    license="MIT",
    license_files="LICENSE.txt",
    packages=["pydiceroller"],
    install_requires=[],
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "pydiceroller=pydiceroller.pydiceroller:pydiceroller"
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
)
